@@ -1,8 +1,8 @@
 /*
  * Copyright (c) 2005 Apple Inc. All rights reserved.
  * Copyright (c) 2005-2006 Paul Guyot <pguyot@kallisys.net>,
+ * Copyright (c) 2006-2018 The MacPorts Project
  * All rights reserved.
- * Copyright (c) 2006-2015 The MacPorts Project
  *
  * @APPLE_BSD_LICENSE_HEADER_START@
  *
@@ -37,6 +37,10 @@
 #include "darwintrace.h"
 #include "sandbox_actions.h"
 
+#ifdef HAVE_STDATOMIC_H
+#include <stdatomic.h>
+#endif
+
 #ifdef HAVE_LIBKERN_OSATOMIC_H
 #include <libkern/OSAtomic.h>
 #endif
@@ -50,7 +54,6 @@
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
-#include <sys/syscall.h>
 #include <sys/types.h>
 #include <sys/un.h>
 #include <unistd.h>
@@ -63,24 +66,6 @@
 #define LSTATSYSNUM SYS_lstat
 #endif
 
-#ifndef HAVE_STRLCPY
-/* Define strlcpy if it's not available. */
-size_t strlcpy(char *dst, const char *src, size_t size) {
-	size_t result = strlen(src);
-	if (size > 0) {
-		size_t copylen = size - 1;
-		if (copylen > result) {
-			copylen = result;
-		}
-		memcpy(dst, src, copylen);
-		dst[copylen] = 0;
-	}
-	return result;
-}
-#endif
-
-#include "../pextlib1.0/strlcat.c"
-
 // Global Variables
 /**
  * PID of the process darwintrace was last used in. This is used to detect
@@ -103,7 +88,6 @@ volatile int __darwintrace_close_sock = -1;
 FILE *__darwintrace_stderr = NULL;
 
 static inline void __darwintrace_log_op(const char *op, const char *path);
-static void __darwintrace_setup_tls() __attribute__((constructor));
 static char *__send(const char *buf, uint32_t len, int answer);
 
 /**
@@ -130,7 +114,32 @@ pthread_key_t sock_key;
  *  2: check for a dependency using the socket
  *  3: deny access to the path and stop processing
  */
+#ifdef HAVE_STDATOMIC_H
+static _Atomic(char *) filemap;
+#else
 static char *filemap;
+#endif
+
+
+volatile bool __darwintrace_initialized = false;
+
+/**
+ * "Constructors" we'd like to run before we do anything. As using
+ * __attribute__((constructor)) for these would be unsafe here (as our
+ * interposed functions might end up being called first) we'll run them manually
+ * before we interpose anything.
+ */
+static void (*constructors[])() = {
+	__darwintrace_setup_tls,
+	__darwintrace_store_env,
+};
+
+void __darwintrace_run_constructors() {
+	for (size_t i = 0; i < sizeof(constructors) / sizeof(*constructors); ++i) {
+		constructors[i]();
+	}
+	__darwintrace_initialized = true;
+}
 
 static void __darwintrace_sock_destructor(FILE *dtsock) {
 	__darwintrace_close_sock = fileno(dtsock);
@@ -143,7 +152,7 @@ static void __darwintrace_sock_destructor(FILE *dtsock) {
  * Setup method called as constructor to set up thread-local storage for the
  * thread id and the darwintrace socket.
  */
-static void __darwintrace_setup_tls() {
+void __darwintrace_setup_tls() {
 	if (0 != (errno = pthread_key_create(&tid_key, NULL))) {
 		perror("darwintrace: pthread_key_create");
 		abort();
@@ -287,17 +296,19 @@ static void __darwintrace_get_filemap() {
 	char *path, command;
 #endif
 
-#if defined(HAVE_OSATOMICCOMPAREANDSWAPPTR)
-#	define CAS(old, new, mem) OSAtomicCompareAndSwapPtr(old, new, (void * volatile *) (mem))
+#if HAVE_DECL_ATOMIC_COMPARE_EXCHANGE_STRONG_EXPLICIT   /* HAVE_DECL_* is always defined and set to 1 or 0 */
+#	define CAS(old, new, mem) atomic_compare_exchange_strong_explicit(mem, old, new, memory_order_relaxed, memory_order_relaxed)
+#elif defined(HAVE_OSATOMICCOMPAREANDSWAPPTR)
+#	define CAS(old, new, mem) OSAtomicCompareAndSwapPtr(*old, new, (void * volatile *) (mem))
 #elif defined(__LP64__)
 #	ifdef HAVE_OSATOMICCOMPAREANDSWAP64
-#		define CAS(old, new, mem) OSAtomicCompareAndSwap64((int64_t) (old), (int64_t) (new), (volatile int64_t *) (mem))
+#		define CAS(old, new, mem) OSAtomicCompareAndSwap64((int64_t) (*old), (int64_t) (new), (volatile int64_t *) (mem))
 #	else
 #		error "No 64-bit compare and swap primitive available on 64-bit OS."
 #	endif
 #else
 #	ifdef HAVE_OSATOMICCOMPAREANDSWAP32
-#		define CAS(old, new, mem) OSAtomicCompareAndSwap32((int32_t) (old), (int32_t) (new), (volatile int32_t *) (mem))
+#		define CAS(old, new, mem) OSAtomicCompareAndSwap32((int32_t) (*old), (int32_t) (new), (volatile int32_t *) (mem))
 #	else
 #		error "No 32-bit compare and swap primitive available."
 #	endif
@@ -312,12 +323,13 @@ static void __darwintrace_get_filemap() {
 	 * code is actually called in an application.
 	 */
 	newfilemap = NULL;
+	char *nullpointer = NULL;
 	do {
 		free(newfilemap);
 		if (filemap != NULL)
 			break;
 		newfilemap = __send("filemap\t", 8, 1);
-	} while (!CAS(NULL, newfilemap, &filemap));
+	} while (!CAS(&nullpointer, newfilemap, &filemap));
 
 #if DARWINTRACE_DEBUG && 0
 	for (__darwintrace_filemap_iterator_init(&it);
@@ -393,7 +405,7 @@ void __darwintrace_setup() {
 		}
 
 		/* Set the close-on-exec flag as early as possible after the socket
-		 * creation. On OS X, there is no way to do this race-condition free
+		 * creation. On macOS, there is no way to do this race-condition free
 		 * unless you synchronize around creation and fork(2) -- however,
 		 * blocking in this function is not acceptable for darwintrace, because
 		 * it could possibly run in a signal handler, leading to a deadlock.
@@ -472,7 +484,6 @@ static inline void __darwintrace_log_op(const char *op, const char *path) {
  *         -1 if MacPorts doesn't know about the file.
  */
 static int dependency_check(const char *path) {
-#define lstat(y, z) syscall(LSTATSYSNUM, (y), (z))
 	char buffer[BUFFER_SIZE], *p;
 	uint32_t len;
 	int result = 0;
@@ -517,7 +528,6 @@ static int dependency_check(const char *path) {
 
 	free(p);
 	return result;
-#undef lstat
 }
 
 /**
@@ -533,7 +543,7 @@ static void frecv(void *restrict buf, size_t size) {
 	 * SA_RESTART isn't set) and fread(3) may return short without giving us
 	 * a way to know how many bytes have actually been read, i.e. without a way
 	 * to do the call again. Because of this great API design and
-	 * implementation on OS X, we'll just use read(2) here. */
+	 * implementation on macOS, we'll just use read(2) here. */
 	int fd = fileno(__darwintrace_sock());
 	size_t count = 0;
 	while (count < size) {
@@ -568,7 +578,7 @@ static void fsend(const void *restrict buf, size_t size) {
 	 * SA_RESTART isn't set) and fwrite(3) may return short without giving us
 	 * a way to know how many bytes have actually been written, i.e. without
 	 * a way to do the call again. Because of this great API design and
-	 * implementation on OS X, we'll just use write(2) here. */
+	 * implementation on macOS, we'll just use write(2) here. */
 	int fd = fileno(__darwintrace_sock());
 	size_t count = 0;
 	while (count < size) {
@@ -613,6 +623,9 @@ static char *__send(const char *buf, uint32_t len, int answer) {
 	}
 
 	recv_buf = malloc(recv_len + 1);
+	if (recv_buf == NULL) {
+		return NULL;
+	}
 	recv_buf[recv_len] = '\0';
 	frecv(recv_buf, recv_len);
 
@@ -638,10 +651,9 @@ static char *__send(const char *buf, uint32_t len, int answer) {
  *         should be denied
  */
 static inline bool __darwintrace_sandbox_check(const char *path, int flags) {
-#define lstat(x,y) syscall(LSTATSYSNUM, (x), (y))
 	filemap_iterator_t filemap_it;
 
-	char command;
+	char command = -1;
 	char *t;
 
 	if (path[0] == '/' && path[1] == '\0') {
@@ -699,7 +711,6 @@ static inline bool __darwintrace_sandbox_check(const char *path, int flags) {
 		__darwintrace_log_op("sandbox_violation", path);
 	}
 	return false;
-#undef lstat
 }
 
 /**
@@ -720,9 +731,6 @@ static inline bool __darwintrace_sandbox_check(const char *path, int flags) {
  *         should be denied
  */
 bool __darwintrace_is_in_sandbox(const char *path, int flags) {
-#define lstat(x, y) syscall(LSTATSYSNUM, (x), (y))
-#define readlink(x,y,z) syscall(SYS_readlink, (x), (y), (z))
-#define getattrlist(v,w,x,y,z) syscall(SYS_getattrlist, (v), (w), (x), (y), (z))
 	if (!filemap) {
 		return true;
 	}
@@ -1031,7 +1039,4 @@ bool __darwintrace_is_in_sandbox(const char *path, int flags) {
 	} while (pathIsSymlink);
 
 	return __darwintrace_sandbox_check(normPath, flags);
-#undef getattrlist
-#undef readlink
-#undef lstat
 }
