@@ -1,11 +1,11 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # diagnose.tcl
 #
-# Copyright (c) 2002 - 2003 Apple Inc.
-# Copyright (c) 2004 - 2005 Paul Guyot, <pguyot@kallisys.net>.
-# Copyright (c) 2004 - 2006 Ole Guldberg Jensen <olegb@opendarwin.org>.
-# Copyright (c) 2004 - 2005 Robert Shaw <rshaw@opendarwin.org>
-# Copyright (c) 2004 - 2014 The MacPorts Project
+# Copyright (c) 2002-2003 Apple Inc.
+# Copyright (c) 2004-2005 Paul Guyot, <pguyot@kallisys.net>.
+# Copyright (c) 2004-2006 Ole Guldberg Jensen <olegb@opendarwin.org>.
+# Copyright (c) 2004-2005 Robert Shaw <rshaw@opendarwin.org>
+# Copyright (c) 2004-2014, 2016-2018 The MacPorts Project
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -91,7 +91,7 @@ namespace eval diagnose {
 
         array set config_options    [list]
         set parser_options          [list macports_location profile_path shell_location \
-                                    xcode_version_${macports::macosx_version} xcode_build]
+                                    xcode_version_${macports::macos_version_major} xcode_build]
 
         set user_config_path        "${macports::autoconf::macports_conf_path}/port_diagnose.ini"
         set xcode_config_path       [macports::getdefaultportresourcepath "macports1.0/xcode_versions.ini"]
@@ -150,9 +150,7 @@ namespace eval diagnose {
 
         output "command line tools"
 
-        set version ${macports::macosx_version}
-
-        if {$version eq "10.9"} {
+        if {${macports::macos_version_major} eq "10.9"} {
 
             if {![file exists "/Library/Developer/CommandLineTools/"]} {
 
@@ -166,7 +164,7 @@ namespace eval diagnose {
 
         } else {
 
-            set xcode_select [exec xcode-select -p]
+            set xcode_select [exec xcode-select -print-path]
 
             if {$xcode_select eq ""} {
 
@@ -326,7 +324,7 @@ namespace eval diagnose {
         output "HomeBrew"
 
         if {[file exists "/usr/local/Cellar"]} {
-            ui_warn "it seems you have Homebrew installed on this system -- Because Homebrew uses /usr/local, this can potentially cause issues \
+            ui_warn "it seems you have Homebrew installed on this system -- Because Homebrew uses /usr/local, this can potentially cause issues\
                      with MacPorts. We'd recommend you either uninstall it, or move it from /usr/local for now."
 
             success_fail 0
@@ -338,7 +336,7 @@ namespace eval diagnose {
 
         output "Fink"
         if {[file exists "/sw"]} {
-            ui_warn "it seems you have Fink installed on your system -- This could potentially cause issues with MacPorts. We'd recommend you \
+            ui_warn "it seems you have Fink installed on your system -- This could potentially cause issues with MacPorts. We'd recommend you\
                      either uninstall it, or move it from /sw for now."
 
             success_fail 0
@@ -458,7 +456,7 @@ namespace eval diagnose {
         output "dylibs in /usr/local/lib"
 
         if {[glob -nocomplain -directory "/usr/local/lib" *.dylib *.la *.prl] ne ""} {
-            ui_warn "found dylibs in your /usr/local/lib directory. These are known to cause problems. We'd recommend \
+            ui_warn "found dylibs in your /usr/local/lib directory. These are known to cause problems. We'd recommend\
                      you remove them."
 
             success_fail 0
@@ -471,7 +469,7 @@ namespace eval diagnose {
         output "header files in /usr/local/include"
 
         if {[glob -nocomplain -directory "/usr/local/include" *.h *.hpp *.hxx] ne ""} {
-            ui_warn "found header files in your /usr/local/include directory. These are known to cause problems. We'd recommend \
+            ui_warn "found header files in your /usr/local/include directory. These are known to cause problems. We'd recommend\
                      you remove them."
 
             success_fail 0
@@ -491,12 +489,12 @@ namespace eval diagnose {
         # Returns:
         #           None
 
-        if {${macports::macosx_version} eq "10.6"} {
-            output "X11.app on OS X 10.6 systems"
+        if {${macports::macos_version_major} eq "10.6"} {
+            output "X11.app on Mac OS X 10.6 systems"
 
             if {[file exists /Applications/X11.app]} {
-                ui_error "it seems you have Mac OS X 10.6 installed, and are using X11 from \"X11.app\". This has been known to cause issues. \
-                         To fix this, please install xorg-server, by using the command 'sudo port install xorg-server', or installing it from \
+                ui_error "it seems you have Mac OS X 10.6 installed, and are using X11 from \"X11.app\". This has been known to cause issues.\
+                         To fix this, please install xorg-server, by using the command 'sudo port install xorg-server', or installing it from\
                          their website, https://www.xquartz.org/releases/."
 
                 success_fail 0
@@ -523,7 +521,7 @@ namespace eval diagnose {
         set available       [lindex $disk_info 3]
 
         if {$available < 5} {
-            ui_warn "you have less than 5 GiB of free disk space! This can cause serious errors. We recommend trying to clear out unnecessary \
+            ui_warn "you have less than 5 GiB of free disk space! This can cause serious errors. We recommend trying to clear out unnecessary\
                      programs and files by running 'sudo port reclaim', or manually uninstalling/deleting programs and folders on your drive."
 
             success_fail 0
@@ -565,7 +563,7 @@ namespace eval diagnose {
         output "for '$app'"
 
         if {![file exists /usr/bin/$app]} {
-            ui_error "$app is needed by MacPorts to function normally, but wasn't found on this system. We'd recommend \
+            ui_error "$app is needed by MacPorts to function normally, but wasn't found on this system. We'd recommend\
                       installing it for continued use of MacPorts."
             success_fail 0
             return
@@ -587,7 +585,7 @@ namespace eval diagnose {
 
         upvar $config_options config
 
-        set mac_version     ${macports::macosx_version}
+        set mac_version     ${macports::macos_version_major}
         set xcode_current   ${macports::xcodeversion}
         if {[info exists config(xcode_version_$mac_version)]} {
             set xcode_versions  $config(xcode_version_$mac_version)
