@@ -1003,21 +1003,21 @@ proc portconfigure::get_min_gcc {} {
 
     set min_value 1.0
     if {${compiler.c_standard} >= 2017} {
-        set min_value [max_version $min_value 8.0]
+        set min_value [max_version $min_value 8]
     } elseif {${compiler.c_standard} >= 2011} {
         set min_value [max_version $min_value 4.3]
     }  elseif {${compiler.c_standard} >= 1999} {
         set min_value [max_version $min_value 4.0]
     }
     if {${compiler.cxx_standard} >= 2017} {
-        set min_value [max_version $min_value 7.0]
+        set min_value [max_version $min_value 7]
     } elseif {${compiler.cxx_standard} >= 2014} {
-        set min_value [max_version $min_value 5.0]
+        set min_value [max_version $min_value 5]
     } elseif {${compiler.cxx_standard} >= 2011} {
-        set min_value [max_version $min_value 4.8.1]
+        set min_value [max_version $min_value 4.8]
     }
     if {[vercmp ${compiler.openmp_version} 4.5] >= 0} {
-        set min_value [max_version $min_value 8.1]
+        set min_value [max_version $min_value 8]
     } elseif {[vercmp ${compiler.openmp_version} 4.0] >= 0} {
         set min_value [max_version $min_value 4.9]
     } elseif {[vercmp ${compiler.openmp_version} 3.1] >= 0} {
@@ -1035,7 +1035,7 @@ proc portconfigure::get_min_gcc {} {
         ([option compiler.limit_flags] || [option compiler.support_environment_sdkroot]) &&
         [option configure.sdkroot] ne ""
     } {
-        set min_value [max_version $min_value 7.0]
+        set min_value [max_version $min_value 7]
     }
     return ${min_value}
 }
@@ -1044,7 +1044,7 @@ proc portconfigure::get_min_gfortran {} {
     global compiler.openmp_version compiler.thread_local_storage
     set min_value 1.0
     if {[vercmp ${compiler.openmp_version} 4.5] >= 0} {
-        set min_value [max_version $min_value 8.1]
+        set min_value [max_version $min_value 8]
     } elseif {[vercmp ${compiler.openmp_version} 4.0] >= 0} {
         set min_value [max_version $min_value 4.9]
     } elseif {[vercmp ${compiler.openmp_version} 3.1] >= 0} {
@@ -1060,7 +1060,7 @@ proc portconfigure::get_min_gfortran {} {
         ([option compiler.limit_flags] || [option compiler.support_environment_sdkroot]) &&
         [option configure.sdkroot] ne ""
     } {
-        set min_value [max_version $min_value 7.0]
+        set min_value [max_version $min_value 7]
     }
     return ${min_value}
 }
@@ -1170,18 +1170,30 @@ proc portconfigure::get_clang_compilers {} {
         source ${compiler_file}
     } else {
         ui_debug "clang_compilers.tcl not found in ports tree, using built-in selections"
-        # clang 9.0 and older build on 10.6+ (darwin 10)
-        # clang 7.0 and older build on 10.5+ (darwin 9)
-        # clang 3.4 and older build on 10.4+ (darwin 8)
+
+        # clang 11  and older build on 10.6+  (darwin 10)
+        # clang 7.0 and older build on 10.5+  (darwin 9)
+        # clang 3.4 and older build on 10.4+  (darwin 8)
+        # Clang 11 and newer only on Apple Silicon
+        # Clang 9.0 and newer only on 11+ (Darwin 20)
+
+        if {${os.major} >= 11} {
+            lappend compilers macports-clang-13 macports-clang-12
+        }
         if {${os.major} >= 10} {
-            lappend compilers macports-clang-9.0 \
-                macports-clang-8.0
+            lappend compilers macports-clang-11
+            if {[option build_arch] ne "arm64"} {
+                lappend compilers macports-clang-10 macports-clang-9.0
+                if {${os.major} < 20} {
+                    lappend compilers macports-clang-8.0
+                }
+            }
         }
 
-        if {${os.major} >= 9} {
+        if {${os.major} >= 9 && ${os.major} < 20} {
             lappend compilers macports-clang-7.0 \
-                macports-clang-6.0 \
-                macports-clang-5.0
+                              macports-clang-6.0 \
+                              macports-clang-5.0
         }
 
         if {${os.major} < 16} {
@@ -1199,18 +1211,30 @@ proc portconfigure::get_clang_compilers {} {
 }
 # utility procedure: get GCC compilers based on os.major
 proc portconfigure::get_gcc_compilers {} {
-    global os.major porturl
+    global os.major os.arch porturl
     set compilers ""
     set compiler_file [getportresourcepath $porturl "port1.0/compilers/gcc_compilers.tcl"]
     if {[file exists ${compiler_file}]} {
         source ${compiler_file}
     } else {
         ui_debug "gcc_compilers.tcl not found in ports tree, using built-in selections"
+
         if {${os.major} >= 10} {
-            # see https://trac.macports.org/ticket/57135
-            lappend compilers macports-gcc-8
+            lappend compilers macports-gcc-11 macports-gcc-10
+        }
+
+        if {${os.arch} ne "arm"} {
+            if {${os.major} >= 10} {
+                lappend compilers macports-gcc-9 macports-gcc-8
+            }
+            if {${os.major} < 20} {
+                lappend compilers macports-gcc-7 macports-gcc-6 macports-gcc-5
+            }
+        }
+
+        if {${os.major} >= 11} {
+            lappend compilers macports-gcc-devel
         }
-        lappend compilers macports-gcc-7 macports-gcc-6 macports-gcc-5
     }
     return ${compilers}
 }
@@ -1592,17 +1616,27 @@ proc portconfigure::add_compiler_port_dependencies {compiler} {
                 source ${dependencies_file}
             } else {
                 ui_debug "gcc_dependencies.tcl not found in ports tree, using built-in data"
+
+                # GCC version providing the primary runtime
+                # Note settings here *must* match those in the lang/libgcc port and compilers PG
+                if {${os.major} < 10} {
+                    set gcc_main_version 7
+                } else {
+                    set gcc_main_version 11
+                }
+
                 # compiler links against libraries in libgcc\d* and/or libgcc-devel
                 if {[vercmp ${gcc_version} 4.6] < 0} {
-                    set libgccs "path:lib/libgcc/libgcc_s.1.dylib:libgcc port:libgcc45"
+                    set libgccs "path:share/doc/libgcc/README:libgcc port:libgcc45"
                 } elseif {[vercmp ${gcc_version} 7] < 0} {
-                    set libgccs "path:lib/libgcc/libgcc_s.1.dylib:libgcc port:libgcc6"
-                } elseif {[vercmp ${gcc_version} 8] < 0} {
-                    set libgccs "path:lib/libgcc/libgcc_s.1.dylib:libgcc port:libgcc7"
-                } elseif {[vercmp ${gcc_version} 9] < 0} {
-                    set libgccs "path:lib/libgcc/libgcc_s.1.dylib:libgcc port:libgcc8"
+                    set libgccs "path:share/doc/libgcc/README:libgcc port:libgcc6"
+                } elseif {[vercmp ${gcc_version} ${gcc_main_version}] < 0} {
+                    set libgccs "path:share/doc/libgcc/README:libgcc port:libgcc${gcc_version}"
                 } else {
-                    set libgccs "path:lib/libgcc/libgcc_s.1.dylib:libgcc"
+                    # Using primary GCC version
+                    # Do not depend directly on primary runtime port, as implied by libgcc
+                    # and doing so prevents libgcc-devel being used as an alternative.
+                    set libgccs "path:share/doc/libgcc/README:libgcc"
                 }
             }
             foreach libgcc_dep $libgccs {
@@ -1679,7 +1713,7 @@ proc portconfigure::configure_main {args} {
         }
 
         parse_environment xmkmf
-        if {[catch {command_exec {*}${callback} "cd ${worksrcpath} && make Makefiles" -varprefix xmkmf} result]} {
+        if {[catch {command_exec {*}${callback} -varprefix xmkmf "cd ${worksrcpath} && make Makefiles"} result]} {
             return -code error "[format [msgcat::mc "%s failure: %s"] "make Makefiles" $result]"
         }
     } elseif {[tbool use_configure]} {
