@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 #
-# Copyright (c) 2004 - 2014, 2016 The MacPorts Project
+# Copyright (c) 2004 - 2014, 2016-2018 The MacPorts Project
 # Copyright (c) 2002 - 2003 Apple Inc.
 # All rights reserved.
 #
@@ -46,9 +46,10 @@ namespace eval portfetch {
 }
 
 # define options: distname master_sites
-options master_sites patch_sites extract.suffix distfiles patchfiles use_bzip2 use_lzma use_xz use_zip use_7z use_lzip use_dmg dist_subdir \
+options master_sites patch_sites extract.suffix distfiles patchfiles use_tar \
+    use_bzip2 use_lzma use_xz use_zip use_7z use_lzip use_dmg dist_subdir \
     fetch.type fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert \
-    master_sites.mirror_subdir patch_sites.mirror_subdir \
+    fetch.user_agent master_sites.mirror_subdir patch_sites.mirror_subdir \
     bzr.url bzr.revision \
     cvs.module cvs.root cvs.password cvs.date cvs.tag cvs.method \
     svn.url svn.revision svn.method \
@@ -67,10 +68,10 @@ default fetch.type standard
 
 default bzr.cmd {[findBinary bzr $portutil::autoconf::bzr_path]}
 default bzr.dir {${workpath}}
-default bzr.revision {-1}
-default bzr.pre_args {"--builtin --no-aliases checkout --lightweight --verbose"}
+default bzr.revision -1
+default bzr.pre_args "--builtin --no-aliases checkout --lightweight --verbose"
 default bzr.args ""
-default bzr.post_args {"-r ${bzr.revision} ${bzr.url} ${worksrcdir}"}
+default bzr.post_args {-r ${bzr.revision} ${bzr.url} ${worksrcdir}}
 
 default cvs.cmd {[findBinary cvs $portutil::autoconf::cvs_path]}
 default cvs.password ""
@@ -80,16 +81,16 @@ default cvs.module {$distname}
 default cvs.tag ""
 default cvs.date ""
 default cvs.env {CVS_PASSFILE=${workpath}/.cvspass}
-default cvs.pre_args {"-z9 -f -d ${cvs.root}"}
+default cvs.pre_args {-z9 -f -d ${cvs.root}}
 default cvs.args ""
-default cvs.post_args {"${cvs.module}"}
+default cvs.post_args {${cvs.module}}
 
-default svn.cmd {[portfetch::find_svn_path]}
+default svn.cmd {${prefix_frozen}/bin/svn}
 default svn.dir {${workpath}}
 default svn.method {export}
 default svn.revision ""
 default svn.env {}
-default svn.pre_args {"--non-interactive --trust-server-cert"}
+default svn.pre_args --non-interactive
 default svn.args ""
 default svn.post_args ""
 
@@ -99,27 +100,29 @@ default git.branch {}
 
 default hg.cmd {[findBinary hg $portutil::autoconf::hg_path]}
 default hg.dir {${workpath}}
-default hg.tag {tip}
+default hg.tag tip
 
 # Set distfiles
-default distfiles {[list [portfetch::suffix $distname]]}
+default distfiles {[list [portfetch::suffix [join $distname]]]}
 default dist_subdir {${name}}
 
 # user name & password
 default fetch.user ""
 default fetch.password ""
 # Use EPSV for FTP transfers
-default fetch.use_epsv "yes"
+default fetch.use_epsv yes
 # Ignore SSL certificate
-default fetch.ignore_sslcert "no"
+default fetch.ignore_sslcert no
 # Use remote timestamps
-default fetch.remote_time "no"
+default fetch.remote_time no
+default fetch.user_agent ""
 
-default global_mirror_site "macports_distfiles"
-default mirror_sites.listfile {"mirror_sites.tcl"}
-default mirror_sites.listpath {"port1.0/fetch"}
+default global_mirror_site macports_distfiles
+default mirror_sites.listfile mirror_sites.tcl
+default mirror_sites.listpath port1.0/fetch
 
 # Option-executed procedures
+option_proc use_tar   portfetch::set_extract_type
 option_proc use_bzip2 portfetch::set_extract_type
 option_proc use_lzma  portfetch::set_extract_type
 option_proc use_xz    portfetch::set_extract_type
@@ -134,6 +137,9 @@ proc portfetch::set_extract_type {option action args} {
     global extract.suffix
     if {[string equal ${action} "set"] && [tbool args]} {
         switch $option {
+            use_tar {
+                set extract.suffix .tar
+            }
             use_bzip2 {
                 set extract.suffix .tar.bz2
                 if {![catch {findBinary lbzip2} result]} {
@@ -181,16 +187,11 @@ proc portfetch::set_fetch_type {option action args} {
                 depends_fetch-append bin:cvs:cvs
             }
             svn {
-                # Sierra is the first macOS version whose svn supports modern TLS cipher suites.
-                if {${os.major} >= 16 || ${os.platform} ne "darwin"} {
-                    depends_fetch-append bin:svn:subversion
-                } else {
-                    depends_fetch-append port:subversion
-                }
+                depends_fetch-append port:subversion
             }
             git {
-                # Mavericks is the first OS X version whose git supports modern TLS cipher suites.
-                if {${os.major} >= 13 || ${os.platform} ne "darwin"} {
+                # Oldest macOS version whose git can validate GitHub's SSL certificate.
+                if {${os.major} >= 14 || ${os.platform} ne "darwin"} {
                     depends_fetch-append bin:git:git
                 } else {
                     depends_fetch-append port:git
@@ -203,23 +204,13 @@ proc portfetch::set_fetch_type {option action args} {
     }
 }
 
-proc portfetch::find_svn_path {args} {
-    global prefix os.platform os.major
-    # Sierra is the first macOS version whose svn supports modern TLS cipher suites.
-    if {${os.major} >= 16 || ${os.platform} ne "darwin"} {
-        return [findBinary svn $portutil::autoconf::svn_path]
-    } else {
-        return ${prefix}/bin/svn
-    }
-}
-
 proc portfetch::find_git_path {args} {
-    global prefix os.platform os.major
-    # Mavericks is the first OS X version whose git supports modern TLS cipher suites.
-    if {${os.major} >= 13 || ${os.platform} ne "darwin"} {
+    global prefix_frozen os.platform os.major
+    # Oldest macOS version whose git can validate GitHub's SSL certificate.
+    if {${os.major} >= 14 || ${os.platform} ne "darwin"} {
         return [findBinary git $portutil::autoconf::git_path]
     } else {
-        return ${prefix}/bin/git
+        return ${prefix_frozen}/bin/git
     }
 }
 
@@ -229,7 +220,7 @@ set_ui_prefix
 # Given a distname, return the distname with extract.suffix appended
 proc portfetch::suffix {distname} {
     global extract.suffix
-    return "${distname}${extract.suffix}"
+    return ${distname}[join ${extract.suffix}]
 }
 # XXX import suffix into the global namespace as it is currently used from
 # Portfiles, but should better go somewhere else
@@ -288,12 +279,18 @@ proc portfetch::get_full_mirror_sites_path {} {
 # Perform the full checksites/checkpatchfiles/checkdistfiles sequence.
 # This method is used by distcheck target.
 proc portfetch::checkfiles {urls} {
-    global global_mirror_site
+    global global_mirror_site ports_fetch_no-mirrors license
     upvar $urls fetch_urls
 
-    checksites [list patch_sites [list $global_mirror_site PATCH_SITE_LOCAL] \
-                master_sites [list $global_mirror_site MASTER_SITE_LOCAL]] \
-               [get_full_mirror_sites_path]
+    set sites [list patch_sites {} \
+                    master_sites {}]
+    if {(![info exists ports_fetch_no-mirrors] || ${ports_fetch_no-mirrors} eq "no") \
+            && [lsearch -exact -nocase $license "nomirror"] == -1} {
+        set sites [list patch_sites [list $global_mirror_site PATCH_SITE_LOCAL] \
+                        master_sites [list $global_mirror_site MASTER_SITE_LOCAL]]
+    }
+
+    checksites $sites [get_full_mirror_sites_path]
     checkpatchfiles fetch_urls
     checkdistfiles fetch_urls
 }
@@ -367,7 +364,7 @@ proc portfetch::cvsfetch {args} {
         set cvs.cmd "echo ${cvs.password} | ${cvs.cmd}"
         set cvs.args login
         set cvs.post_args ""
-        if {[catch {command_exec cvs -notty "" "2>&1"} result]} {
+        if {[catch {command_exec -notty cvs "" "2>&1"} result]} {
             return -code error [msgcat::mc "CVS login failed"]
         }
         set cvs.cmd ${savecmd}
@@ -449,14 +446,14 @@ proc portfetch::gitfetch {args} {
         # if we're just using HEAD, we can make a shallow repo
         append options " --depth=1"
     }
-    set cmdstring "${git.cmd} clone $options ${git.url} ${worksrcpath} 2>&1"
+    set cmdstring "${git.cmd} clone $options ${git.url} [shellescape ${worksrcpath}] 2>&1"
     ui_debug "Executing: $cmdstring"
     if {[catch {system $cmdstring} result]} {
         return -code error [msgcat::mc "Git clone failed"]
     }
 
     if {${git.branch} ne ""} {
-        set env "GIT_DIR=${worksrcpath}/.git GIT_WORK_TREE=${worksrcpath}"
+        set env "GIT_DIR=[shellescape ${worksrcpath}/.git] GIT_WORK_TREE=[shellescape ${worksrcpath}]"
         set cmdstring "$env ${git.cmd} checkout -q ${git.branch} 2>&1"
         ui_debug "Executing $cmdstring"
         if {[catch {system $cmdstring} result]} {
@@ -473,7 +470,7 @@ proc portfetch::gitfetch {args} {
 
 # Perform a mercurial fetch.
 proc portfetch::hgfetch {args} {
-    global worksrcpath prefix_frozen patchfiles hg.url hg.tag hg.cmd \
+    global worksrcpath patchfiles hg.url hg.tag hg.cmd \
            fetch.ignore_sslcert
 
     set insecureflag ""
@@ -481,7 +478,7 @@ proc portfetch::hgfetch {args} {
         set insecureflag " --insecure"
     }
 
-    set cmdstring "${hg.cmd} clone${insecureflag} --rev \"${hg.tag}\" ${hg.url} ${worksrcpath} 2>&1"
+    set cmdstring "${hg.cmd} clone${insecureflag} --rev \"${hg.tag}\" ${hg.url} [shellescape ${worksrcpath}] 2>&1"
     ui_debug "Executing: $cmdstring"
     if {[catch {system $cmdstring} result]} {
         return -code error [msgcat::mc "Mercurial clone failed"]
@@ -499,11 +496,11 @@ proc portfetch::hgfetch {args} {
 proc portfetch::fetchfiles {args} {
     global distpath all_dist_files UI_PREFIX \
            fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert fetch.remote_time \
-           portverbose
+           fetch.user_agent portverbose
     variable fetch_urls
     variable urlmap
 
-    set fetch_options {}
+    set fetch_options [list]
     if {[string length ${fetch.user}] || [string length ${fetch.password}]} {
         lappend fetch_options -u
         lappend fetch_options "${fetch.user}:${fetch.password}"
@@ -517,10 +514,14 @@ proc portfetch::fetchfiles {args} {
     if {${fetch.remote_time} ne "no"} {
         lappend fetch_options "--remote-time"
     }
+    if {${fetch.user_agent} ne ""} {
+        lappend fetch_options "--user-agent"
+        lappend fetch_options "${fetch.user_agent}"
+    }
     if {$portverbose eq "yes"} {
         lappend fetch_options "--progress"
         lappend fetch_options "builtin"
-    } elseif {[llength [info commands ui_progress_download]] > 0} {
+    } else {
         lappend fetch_options "--progress"
         lappend fetch_options "ui_progress_download"
     }
